/* Program to generate data from test functions used by J.-N. Hwang, et al. 

   Written by Radford M. Neal, April 1997.

   This program writes data generated from these functions to standard
   output, in DELVE format.  It also creates files g1.plt, ..., g5.plt
   containing plots of the functions.  Finally, it writes the mean and
   variance of the functions over two different grids to standard error.

   Reference:

   Hwang, J.-N., Lay, S.-R., Maechler, M., Martin, R. D., and 
     Schimert, J. (1994) "Regression modeling in back-propagation 
     and projection pursuit learning", IEEE Transactions on Neural 
     Networks, vol. 5, no. 3, pp. 342-353.

   See the Notes file in this directory for more details.
*/   

#include <stdlib.h>
#include <math.h>
#include <stdio.h>

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif


/* Adjustable parameters */

#define Seed 1		/* Random number seed to use */
#define N_cases 13600	/* Number of cases to generate */


/* Generate number picked from uniform distribution on [0,1). */

double ranu(void)                             
{
  return drand48();
}


/* Generate random value from standard normal distribution. */

double rann(void)
{
  double a, b;
  
  a = ranu();
  b = ranu();
  return cos(2.0*M_PI*a)*sqrt(-2.0*log(b));
}


/* Procedures defining the five functions. */

double g1 (double x1, double x2)
{
  return 10.391 * ((x1-0.4)*(x2-0.6)+0.36);
}

double g2 (double x1, double x2)
{
  double rsq;

  rsq = (x1-0.5)*(x1-0.5) + (x2-0.5)*(x2-0.5);
  return 24.234 * rsq*(0.75-rsq);
}

double g3 (double x1, double x2)
{
  double zr, zi, zpr, zpi, or, oi;
  int j;

  x1 -= 0.5;
  x2 -= 0.5;

  return 42.659  
           * (0.1 + x1 * (0.05 + x1*x1*x1*x1 - 10*x1*x1*x2*x2 + 5*x2*x2*x2*x2));

# if 0 /* Definition in terms of complex numbers - NOT in fact the same. */

  zr = x1-0.5;
  zi = x2-0.5;
   
  zpr = zr;
  zpi = zi;

  for (j = 1; j<5; j++)
  { or = zpr;
    oi = zpi;
    zpr = or*zr - oi*zi;
    zpi = oi*zr + or*zi;
  }

  return 42.659 * ((2+x1)/20 + zpr);

# endif
}

double g4 (double x1, double x2)
{
  return 1.3356 * (1.5*(1-x1) + exp(2*x1-1)*sin(3*M_PI*(x1-0.6)*(x1-0.6))
                              + exp(3*(x2-0.5))*sin(4*M_PI*(x2-0.9)*(x2-0.9)));

}

double g5 (double x1, double x2)
{
  return 1.9 * (1.35 + exp(x1)*sin(13*(x1-0.6)*(x1-0.6))*exp(-x2)*sin(7*x2));
}


/* MAIN PROGRAM. */

main(void)
{
  double m1, m2, m3, m4, m5, v1, v2, v3, v4, v5;
  double x1, x2;
  FILE *plt;
  int i, j;

  /* Write the data file to standard output. */

  srand48(Seed);           

  for (i = 0; i<N_cases; i++) 
  {
    x1 = ranu();          
    x2 = ranu();

    if (g1(x1,x2)<0 || g2(x1,x2)<0 || g3(x1,x2)<0 || g4(x1,x2)<0 || g5(x1,x2)<0)
    { fprintf(stderr,"Function value less than zero encountered at %.3f %.3f\n",
              x1, x2);
    }

    printf("%8.6lf %8.6lf %+10.6lf %+10.6lf %+10.6lf %+10.6lf %+10.6lf \\\n",
      x1, x2, g1(x1,x2), g2(x1,x2), g3(x1,x2), g4(x1,x2), g5(x1,x2)
    );
    printf("                  %+10.6lf %+10.6lf %+10.6lf %+10.6lf %+10.6lf\n",
      g1(x1,x2) + 0.25*rann(),
      g2(x1,x2) + 0.25*rann(),
      g3(x1,x2) + 0.25*rann(),
      g4(x1,x2) + 0.25*rann(),
      g5(x1,x2) + 0.25*rann()
    );
  }

  /* Create the five plot files. */

  plt = fopen("g1.plt","w");

  for (i = 0; i<=40; i++)
  { for (j = 0; j<=40; j++)
    { x1 = i/40.0;
      x2 = j/40.0;
      fprintf(plt,"%.3lf %.3lf  %+10.6lf\n",x1,x2,g1(x1,x2));
    }
    fprintf(plt,"\n");
  }

  fclose(plt);

  plt = fopen("g2.plt","w");

  for (i = 0; i<=40; i++)
  { for (j = 0; j<=40; j++)
    { x1 = i/40.0;
      x2 = j/40.0;
      fprintf(plt,"%.3lf %.3lf  %+10.6lf\n",x1,x2,g2(x1,x2));
    }
    fprintf(plt,"\n");
  }

  fclose(plt);

  plt = fopen("g3.plt","w");

  for (i = 0; i<=40; i++)
  { for (j = 0; j<=40; j++)
    { x1 = i/40.0;
      x2 = j/40.0;
      fprintf(plt,"%.3lf %.3lf  %+10.6lf\n",x1,x2,g3(x1,x2));
    }
    fprintf(plt,"\n");
  }

  fclose(plt);

  plt = fopen("g4.plt","w");

  for (i = 0; i<=40; i++)
  { for (j = 0; j<=40; j++)
    { x1 = i/40.0;
      x2 = j/40.0;
      fprintf(plt,"%.3lf %.3lf  %+10.6lf\n",x1,x2,g4(x1,x2));
    }
    fprintf(plt,"\n");
  }

  fclose(plt);

  plt = fopen("g5.plt","w");

  for (i = 0; i<=40; i++)
  { for (j = 0; j<=40; j++)
    { x1 = i/40.0;
      x2 = j/40.0;
      fprintf(plt,"%.3lf %.3lf  %+10.6lf\n",x1,x2,g5(x1,x2));
    }
    fprintf(plt,"\n");
  }

  fclose(plt);

  /* Compute statistics on a grid and write to standard error. */

  m1 = m2 = m3 = m4 = m5 = 0;
  
  for (i = 1; i<=50; i++)
  { for (j = 1; j<=50; j++)
    { x1 = (i-0.5)/50;
      x2 = (j-0.5)/50;
      m1 += g1(x1,x2);
      m2 += g2(x1,x2);
      m3 += g3(x1,x2);
      m4 += g4(x1,x2);
      m5 += g5(x1,x2);
    }
  }

  m1 /= 50*50; m2 /= 50*50; m3 /= 50*50; m4 /= 50*50; m5 /= 50*50;

  v1 = v2 = v3 = v4 = v5 = 0;
  
  for (i = 1; i<=50; i++)
  { for (j = 1; j<=50; j++)
    { x1 = (i-0.5)/50;
      x2 = (j-0.5)/50;
      v1 += pow (g1(x1,x2) - m1, 2.0);
      v2 += pow (g2(x1,x2) - m2, 2.0);
      v3 += pow (g3(x1,x2) - m3, 2.0);
      v4 += pow (g4(x1,x2) - m4, 2.0);
      v5 += pow (g5(x1,x2) - m5, 2.0);
    }
  }

  v1 /= 50*50; v2 /= 50*50; v3 /= 50*50; v4 /= 50*50; v5 /= 50*50;

  fprintf(stderr,"Statistics from 50x50 grid:\n");
  fprintf(stderr,"        Means: %6.4f %6.4f %6.4f %6.4f %6.4f\n",
                  m1,m2,m3,m4,m5);
  fprintf(stderr,"    Variances: %6.4f %6.4f %6.4f %6.4f %6.4f\n",
                  v1,v2,v3,v4,v5);
  fprintf(stderr,"  1/Variances: %6.4f %6.4f %6.4f %6.4f %6.4f\n",
                  1/v1,1/v2,1/v3,1/v4,1/v5);

  m1 = m2 = m3 = m4 = m5 = 0;
  
  for (i = 1; i<=100; i++)
  { for (j = 1; j<=100; j++)
    { x1 = (i-0.5)/100;
      x2 = (j-0.5)/100;
      m1 += g1(x1,x2);
      m2 += g2(x1,x2);
      m3 += g3(x1,x2);
      m4 += g4(x1,x2);
      m5 += g5(x1,x2);
    }
  }

  m1 /= 100*100; m2 /= 100*100; m3 /= 100*100; m4 /= 100*100; m5 /= 100*100;

  v1 = v2 = v3 = v4 = v5 = 0;
  
  for (i = 1; i<=100; i++)
  { for (j = 1; j<=100; j++)
    { x1 = (i-0.5)/100;
      x2 = (j-0.5)/100;
      v1 += pow (g1(x1,x2) - m1, 2.0);
      v2 += pow (g2(x1,x2) - m2, 2.0);
      v3 += pow (g3(x1,x2) - m3, 2.0);
      v4 += pow (g4(x1,x2) - m4, 2.0);
      v5 += pow (g5(x1,x2) - m5, 2.0);
    }
  }

  v1 /= 100*100; v2 /= 100*100; v3 /= 100*100; v4 /= 100*100; v5 /= 100*100;

  fprintf(stderr,"Statistics from 100x100 grid:\n");
  fprintf(stderr,"        Means: %6.4f %6.4f %6.4f %6.4f %6.4f\n",
                  m1,m2,m3,m4,m5);
  fprintf(stderr,"    Variances: %6.4f %6.4f %6.4f %6.4f %6.4f\n",
                  v1,v2,v3,v4,v5);
  fprintf(stderr,"  1/Variances: %6.4f %6.4f %6.4f %6.4f %6.4f\n",
                  1/v1,1/v2,1/v3,1/v4,1/v5);
 
  exit(0);
}
